/*-----------------------------------------------------------------------------
 *      Name:         DV_Ethernet.c
 *      Purpose:      Ethernet test cases
 *----------------------------------------------------------------------------
 *      Copyright(c) KEIL - An ARM Company
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h" 
#include "cmsis_dv.h" 
#include "DV_Config.h"
#include "DV_Framework.h"
#include "RTE_Device.h" 
#include "Driver_ETH_MAC.h"
#include "Driver_ETH_PHY.h"
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 

// Register Driver_ETH_MAC# Driver_ETH_PHY#
extern ARM_DRIVER_ETH_MAC CREATE_SYMBOL(Driver_ETH_MAC, DRV_ETH);
extern ARM_DRIVER_ETH_PHY CREATE_SYMBOL(Driver_ETH_PHY, DRV_ETH);
static ARM_DRIVER_ETH_MAC* eth_mac = &CREATE_SYMBOL(Driver_ETH_MAC, DRV_ETH);
static ARM_DRIVER_ETH_PHY* eth_phy = &CREATE_SYMBOL(Driver_ETH_PHY, DRV_ETH);
static ARM_ETH_MAC_CAPABILITIES capab;  

// Event flags
static uint8_t volatile Event; 

// Ethernet event
static void ETH_DrvEvent (uint32_t event) {
  if (event & ARM_ETH_MAC_EVENT_RX_FRAME) {
    Event |= 0x01;
  }
}

// Ethernet transfer
int8_t ETH_RunTransfer (uint8_t *out, uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~0x01;     

  eth_mac->SendFrame (out, cnt, 0);
    
  tick = osKernelSysTick();
  do {
    if ((Event & 0x01) || (eth_mac->GetRxFrameSize() != 0)) {
      eth_mac->ReadFrame (in, BUFFER[BUFFER_NUM-1]);     
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(ETH_TRANSFER_TIMEOUT));
  
  return -1;
}


/*-----------------------------------------------------------------------------
 *      Test cases
 *----------------------------------------------------------------------------*/

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\defgroup eth_funcs Ethernet Validation
\brief Ethernet test cases
\details
The Ethernet validation test performs the following checks:
- API interface compliance.
- Data communication with various transfer sizes and communication parameters.
- Transfer speed of the data communication.
- Loopback communication.

\anchor eth_loopback
Loopback Communication Setup
----------------------------

To perform loopback communication tests, it is required to connect the RX and TX lines of the Ethernet cable together:

- TX+ (Pin 1) with RX+ (Pin 3) and
- TX- (Pin 2) with RX- (Pin 6)

\image html ethernet_loopback.png

Various \b Ethernet \b loopback \b plugs are available from different vendors that fulfill this purpose.

@{
*/

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_GetCapabilities
\details
The test case \b ETH_MAC_GetCapabilities verifies the Ethernet MAC function \b GetCapabilities.
*/
void ETH_MAC_GetCapabilities (void) {                    
  /* Get ETH_MAC capabilities */
  capab = eth_mac->GetCapabilities();
  ASSERT_TRUE(&capab != NULL); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_Initialization
\details
The test case \b ETH_MAC_Initialization verifies the Ethernet MAC functions in the following order:
  - \b Initialize  without callback
  - \b Uninitialize
  - \b Initialize with callback if supported
*/
void ETH_MAC_Initialization (void) { 

  /* Initialize without callback */
  ASSERT_TRUE(eth_mac->Initialize(NULL) == ARM_DRIVER_OK); 
    
  /* Uninitialize */
  ASSERT_TRUE(eth_mac->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Initialize with callback if supported */
  ASSERT_TRUE(eth_mac->Initialize((capab.event_rx_frame) ? ETH_DrvEvent : NULL) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_CheckInvalidInit
\details
The test case \b ETH_MAC_CheckInvalidInit verifies the driver behaviour when receiving an invalid initialization sequence:
  - \b Uninitialize
  - \b PowerControl with Power off
  - \b PowerControl with Power on
  - \b Control 
*/
void ETH_MAC_CheckInvalidInit (void) { 

  /* Uninitialize */
  ASSERT_TRUE(eth_mac->Uninitialize() == ARM_DRIVER_OK); 

  /* Power off */
  ASSERT_TRUE(eth_mac->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Try to power on */
  ASSERT_TRUE(eth_mac->PowerControl (ARM_POWER_FULL) != ARM_DRIVER_OK); 
  
  /* Try to set configuration */
  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_SPEED_100M  | ARM_ETH_MAC_DUPLEX_FULL | 
    ARM_ETH_MAC_ADDRESS_BROADCAST | ARM_ETH_MAC_ADDRESS_ALL )!= ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_PowerControl
\details
The test case \b ETH_MAC_PowerControl verifies the Ethernet MAC \b PowerControl function with the sequence:
  - Set bus speed \token{10M}
  - Set bus speed \token{100M}
  - Set bus speed \token{1G} 
*/
void ETH_MAC_PowerControl (void) { 
  int32_t val;
  
  /* Power on */
  ASSERT_TRUE(eth_mac->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);  
  
  /* Power low */
  val = eth_mac->PowerControl (ARM_POWER_LOW);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Low power is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
   
  /* Power off */
  ASSERT_TRUE(eth_mac->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Power on */
  ASSERT_TRUE(eth_mac->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_SetBusSpeed
\details
The test case \b ETH_MAC_SetBusSpeed verifies the Ethernet MAC \b Control function with the sequence:
  - Set bus speed \token{10M}
  - Set bus speed \token{100M}
  - Set bus speed \token{1G}
*/
void ETH_MAC_SetBusSpeed (void) { 
  int32_t val;  
  
  /* Set bus speed 10M */
  val = eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_SPEED_10M);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Link speed 10M is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  /* Set bus speed 100M */
  val = eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_SPEED_100M);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Link speed 100M is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  /* Set bus speed 1G */
  val = eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_SPEED_1G);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Link speed 1G is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_Config_Mode
\details
The test case \b ETH_MAC_Config_Mode verifies the Ethernet MAC \b Control function with the sequence:
  - Set full duplex
  - Set half duplex 
*/
void ETH_MAC_Config_Mode (void) { 
  
  /* Set full duplex */
  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_MAC_DUPLEX_FULL)== ARM_DRIVER_OK);
  
  /* Set half duplex */
  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_MAC_DUPLEX_HALF)== ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_MAC_Config_CommonParams
\details
The test case \b ETH_MAC_Config_CommonParams verifies the Ethernet MAC \b Control function with the sequence:
  - Configure Ethernet MAC bus
  - Configure receiver
  - Configure transmitter
*/
void ETH_MAC_Config_CommonParams (void) { 
  
  /* Configure ETH_MAC bus*/
  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONFIGURE, ARM_ETH_SPEED_100M  | ARM_ETH_MAC_DUPLEX_FULL | 
    ARM_ETH_MAC_ADDRESS_BROADCAST | ARM_ETH_MAC_ADDRESS_ALL | ARM_ETH_MAC_LOOPBACK) == ARM_DRIVER_OK);
 
  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONTROL_RX, 1) == ARM_DRIVER_OK);

  ASSERT_TRUE(eth_mac->Control (ARM_ETH_MAC_CONTROL_TX, 1) == ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_PHY_Initialization
\details
The test case \b ETH_PHY_Initialization verifies the Ethernet PHY functions in the following order:
  - \b Initialize with read and write functions
*/
void ETH_PHY_Initialization (void) { 

  /* Initialize */
  ASSERT_TRUE(eth_phy->Initialize(eth_mac->PHY_Read, eth_mac->PHY_Write) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_PHY_CheckInvalidInit
\details
The test case \b ETH_PHY_CheckInvalidInit verifies the driver behaviour when receiving an invalid initialization sequence:
  - \b Uninitialize
  - \b PowerControl with Power off
  - \b PowerControl with Power on
  - \b SetInterface to configure the Ehternet PHY bus
  - \b SetMode to configure the Ehternet PHY bus
  - \b Initialize  without read and write functions
*/
void ETH_PHY_CheckInvalidInit (void) { 
    
  /* Uninitialize */
  ASSERT_TRUE(eth_phy->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Power off */
  ASSERT_TRUE(eth_phy->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Try to power on*/
  ASSERT_TRUE(eth_phy->PowerControl (ARM_POWER_FULL) != ARM_DRIVER_OK); 
  
  /* Try to configure ETH_PHY bus*/
  ASSERT_TRUE(eth_phy->SetInterface (capab.media_interface) != ARM_DRIVER_OK);  
  ASSERT_TRUE(eth_phy->SetMode (ARM_ETH_PHY_AUTO_NEGOTIATE) != ARM_DRIVER_OK);
    
  /* Try to initialize without read and write functions */
  ASSERT_TRUE(eth_phy->Initialize(NULL, NULL) != ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: ETH_PHY_PowerControl
\details
The test case \b ETH_PHY_PowerControl verifies the Ethernet PHY \b PowerControl function with the sequence:
  - Power on
  - Power low
  - Power off
  - Power on
*/
void ETH_PHY_PowerControl (void) { 
  int32_t val;
  
  /* Power on */
  ASSERT_TRUE(eth_phy->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);  
  
  /* Power low */
  val = eth_phy->PowerControl (ARM_POWER_LOW);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Low power is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
   
  /* Power off */
  ASSERT_TRUE(eth_phy->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Power on */
  ASSERT_TRUE(eth_phy->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: ETH_PHY_Config
\details
The test case \b ETH_PHY_Config verifies the PHY functions 
 - \b SetInterface 
 - \b SetMode 
*/
void ETH_PHY_Config (void) { 
  
  /* Configure ETH_PHY bus*/
  ASSERT_TRUE(eth_phy->SetInterface (capab.media_interface) == ARM_DRIVER_OK);
  
  ASSERT_TRUE(eth_phy->SetMode (ARM_ETH_PHY_AUTO_NEGOTIATE) == ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: ETH_Loopback_Transfer
\details
The test case \b ETH_Loopback_Transfer verifies data transfer via Ehernet with the following sequence:
 - Buffer allocation
 - Ethernet connection
 - Set output buffer pattern
 - Transfer data chunks
 - Set output buffer with random data
 - Transfer data chunks
*/
void ETH_Loopback_Transfer (void) { 
  uint16_t cnt, i; 
  uint8_t pattern[] = BUFFER_PATTERN;
  uint32_t tick;
  char str[64];
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  buffer_in = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);
  
  /* Check Ethernet link*/
  tick = osKernelSysTick();
  while (eth_phy->GetLinkState() != ARM_ETH_LINK_UP) {
    if ((osKernelSysTick() - tick) >= osKernelSysTickMicroSec(ETH_LINK_TIMEOUT)) {
      SET_RESULT(FAILED, "Link is broken, connect Ethernet cable");
      break;
    }
  }   
  
  /* Set output buffer pattern*/
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1];) {  
    for (i = 0; i<ARRAY_SIZE(pattern); i++) {
      buffer_out[cnt++] = pattern[i];
    }
  } 
 
  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer*/
    memset(buffer_in,0,BUFFER[cnt]);    
    if (ETH_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 
  
  /* Set output buffer with random data*/
  srand(osKernelSysTick());
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1]; cnt++) {  
    buffer_out[cnt] = rand();
  }   
  
  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer*/
    memset(buffer_in,0,BUFFER[cnt]);    
    if (ETH_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 
}

/**
@}
*/ 
// end of group eth_funcs
