/*-----------------------------------------------------------------------------
 *      Name:         DV_SPI.c
 *      Purpose:      SPI test cases
 *----------------------------------------------------------------------------
 *      Copyright(c) KEIL - An ARM Company
 *----------------------------------------------------------------------------*/
#include "cmsis_os.h" 
#include "cmsis_dv.h" 
#include "DV_Config.h"
#include "DV_Framework.h"
#include "RTE_Device.h" 
#include "Driver_SPI.h"
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 

// SPI baudrates (kHz)
const uint32_t SPI_BR[] =  {
#if (SPI_BR_1>0)
  SPI_BR_1,
#endif  
#if (SPI_BR_2>0)
  SPI_BR_2,
#endif 
#if (SPI_BR_3>0)
  SPI_BR_3,
#endif 
#if (SPI_BR_4>0)
  SPI_BR_4,
#endif 
#if (SPI_BR_5>0)
  SPI_BR_5,
#endif 
#if (SPI_BR_6>0)
  SPI_BR_6,
#endif 
};
const uint32_t SPI_BR_NUM = ARRAY_SIZE(SPI_BR);
  
// Register Driver_SPI#
extern ARM_DRIVER_SPI CREATE_SYMBOL(Driver_SPI, DRV_SPI);
static ARM_DRIVER_SPI *drv = &CREATE_SYMBOL(Driver_SPI, DRV_SPI);
static ARM_SPI_CAPABILITIES capab;   

// Event flags
static uint8_t volatile Event;    

// SPI event
static void SPI_DrvEvent (uint32_t event) {
  Event |= event;
}

// SPI transfer
int8_t SPI_RunTransfer (uint8_t *out, uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_SPI_EVENT_TRANSFER_COMPLETE;
  drv->Transfer (out, in, cnt);

  tick = osKernelSysTick();
  do {
    if (Event & ARM_SPI_EVENT_TRANSFER_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(SPI_TRANSFER_TIMEOUT));
  return -1;
}

// SPI send
int8_t SPI_RunSend (uint8_t *out, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_SPI_EVENT_TRANSFER_COMPLETE;
  drv->Send (out, cnt);

  tick = osKernelSysTick();
  do {
    if (Event & ARM_SPI_EVENT_TRANSFER_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(SPI_TRANSFER_TIMEOUT));
  return -1;
}

// SPI send without callback
int8_t SPI_RunSend_NoCallback (uint8_t *out, uint32_t cnt) {
  uint32_t tick;
    
  drv->Send (out, cnt);

  tick = osKernelSysTick();
  do {
    if (drv->GetDataCount() == cnt) {
      return 0;
    } 
    if (drv->GetStatus().data_lost) {
      return -1;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(SPI_TRANSFER_TIMEOUT));
  return -1;
}

// SPI receive
int8_t SPI_RunReceive (uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_SPI_EVENT_TRANSFER_COMPLETE;
  drv->Receive (in, cnt);

  tick = osKernelSysTick();
  do {
    if (Event & ARM_SPI_EVENT_TRANSFER_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(SPI_TRANSFER_TIMEOUT));
  return -1;
}

// SPI receive
int8_t SPI_RunReceiveNoCallback (uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~0x01;
  drv->Receive (in, cnt);

  tick = osKernelSysTick();
  do {
    if (drv->GetDataCount() == cnt) {
      return 0;
    } 
    if (drv->GetStatus().data_lost) {
      return -1;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(SPI_TRANSFER_TIMEOUT));
  return -1;
}

/*-----------------------------------------------------------------------------
 *      Test cases
 *----------------------------------------------------------------------------*/
 
/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\defgroup spi_funcs SPI Validation
\brief SPI test cases
\details
The SPI validation test performs the following checks:
- API interface compliance.
- Data communication with various transfer sizes and communication parameters.
- Transfer speed of the data communication.
- Loopback communication.

\anchor spi_loop_back_setup
Loopback Communication Setup
----------------------------

To perform loopback communication tests, it is required to connect the SPI's \b MISO signal to the \b MOSI signal (refer to
the schematics of your target hardware for detailed pinout information).
@{
*/

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: SPI_GetCapabilities
\details
The test case \b SPI_GetCapabilities verifies the function \b GetCapabilities.
*/
void SPI_GetCapabilities (void) {                    
  /* Get SPI capabilities */
  capab = drv->GetCapabilities();
  ASSERT_TRUE(&capab != NULL); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Initialization
\details
The test case \b SPI_Initialization verifies the SPI functions with the sequence:
  - \b Initialize without callback
  - \b Uninitialize
  - \b Initialize with callback
*/
void SPI_Initialization (void) { 
    
  /* Initialize without callback */
  ASSERT_TRUE(drv->Initialize(NULL) == ARM_DRIVER_OK); 
    
  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Initialize with callback */
  ASSERT_TRUE(drv->Initialize(SPI_DrvEvent) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_CheckInvalidInit
\details
The test case \b SPI_CheckInvalidInit verifies the driver behaviour when receiving an invalid initialization sequence:
  - \b Uninitialize
  - \b PowerControl with Power off
  - \b PowerControl with Power on
  - \b Control 
*/
void SPI_CheckInvalidInit (void) { 
  
  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Try to power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) != ARM_DRIVER_OK); 
  
  /* Try to set configuration */
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL0_CPHA0 | ARM_SPI_DATA_BITS(8) | ARM_SPI_MSB_LSB | ARM_SPI_SS_MASTER_UNUSED, 0)
    != ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_PowerControl
\details
The test case \b SPI_PowerControl verifies the \b PowerControl function with the sequence:
 - Power on
 - Power low
 - Power off
 - Power on 
*/
void SPI_PowerControl (void) { 
  int32_t val;
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);  
  
  /* Power low */
  val = drv->PowerControl (ARM_POWER_LOW);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Low power is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
   
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_PolarityPhase
\details
The test case \b SPI_Config_PolarityPhase verifies the \b Control function with the sequence:
 - Set basic SPI bus configuration
 - Change polarity
 - Change phase 
 - Change polarity and phase
*/
void SPI_Config_PolarityPhase (void) { 
  
  /* Set basic SPI bus configuration*/
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL0_CPHA0 | ARM_SPI_DATA_BITS(SPI_DATA_BITS), 0) == ARM_DRIVER_OK);
  
  /* Change polarity */
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL1_CPHA0 | ARM_SPI_DATA_BITS(SPI_DATA_BITS), 0) == ARM_DRIVER_OK);
  
  /* Change phase */
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL0_CPHA1 | ARM_SPI_DATA_BITS(SPI_DATA_BITS), 0) == ARM_DRIVER_OK);
  
  /* Change polarity and phase */
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL1_CPHA1 | ARM_SPI_DATA_BITS(SPI_DATA_BITS), 0) == ARM_DRIVER_OK);
  
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_DataBits
\details
The test case \b SPI_Config_DataBits verifies the \b Control function with the sequence:
 - Data bits = \token{8}
 - Data bits = \token{16}
*/
void SPI_Config_DataBits (void) { 
  int32_t val;  
  
  /* Data bits = 8 */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8), 0);
  if (val == ARM_SPI_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 8 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  /* Data bits = 16 */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(16), 0);
  if (val == ARM_SPI_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 16 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_BitOrder
\details
The test case \b SPI_Config_BitOrder verifies the \b Control function with the sequence:
 - Bit order LSB_MSB
 - Bit order MSB_LSB
*/
void SPI_Config_BitOrder (void) { 
  int32_t val;  
  
  /* Bit order LSB_MSB */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8) | ARM_SPI_LSB_MSB, 0);
  if (val == ARM_SPI_ERROR_BIT_ORDER) { SET_RESULT(WARNING, "Bit order LSB_MSB is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
  
  /* Bit order MSB_LSB */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8) | ARM_SPI_MSB_LSB, 0);
  if (val == ARM_SPI_ERROR_BIT_ORDER) { SET_RESULT(WARNING, "Bit order MSB_LSB is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_SSMode
\details
The test case \b SPI_Config_SSMode verifies the \b Control function with the sequence:
 - SS Mode MASTER_HW_OUTPUT
 - SS Mode MASTER_HW_INPUT 
 - SS Mode MASTER_SW
*/
void SPI_Config_SSMode (void) { 
  int32_t val;  

  /* SS Mode MASTER_HW_OUTPUT */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8) | ARM_SPI_SS_MASTER_HW_OUTPUT, 0);
  if (val == ARM_SPI_ERROR_SS_MODE) { SET_RESULT(WARNING, "Slave select MASTER_HW_OUTPUT is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }

  /* SS Mode MASTER_HW_INPUT */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8) | ARM_SPI_SS_MASTER_HW_INPUT, 0);
  if (val == ARM_SPI_ERROR_SS_MODE) { SET_RESULT(WARNING, "Slave select MASTER_HW_INPUT is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
  
  /* SS Mode MASTER_SW */
  val = drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_DATA_BITS(8) | ARM_SPI_SS_MASTER_SW, 0);
  if (val == ARM_SPI_ERROR_SS_MODE) { SET_RESULT(WARNING, "Slave select MASTER_SW is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_CommonParams
\details
The test case \b SPI_Config_CommonParams verifies the \b Control function with the sequence:
 - Configure SPI bus
*/
void  SPI_Config_CommonParams (void) {
  
  /* Configure SPI bus*/
  ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL0_CPHA0 | ARM_SPI_DATA_BITS(SPI_DATA_BITS) |
  ARM_SPI_MSB_LSB | ARM_SPI_SS_MASTER_UNUSED, SPI_BR[0]*1000) == ARM_DRIVER_OK);
}
  
/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Config_BusSpeed
\details
The test case \b SPI_Config_BusSpeed verifies the \b Control function and tests bus speeds with the sequence:
 - Change bus speed with specific control parameter
 - Read bus speed
 - Change bus speed with general control parameter
 - Read bus speed
*/
void SPI_Config_BusSpeed (void) { 
  int32_t val;
  uint32_t speed;   
  char str[64];
  
  /* Test bus speeds */
  for (speed=0; speed<SPI_BR_NUM; speed++) {

    /* Change bus speed with specific control parameter*/
    ASSERT_TRUE (drv->Control(ARM_SPI_SET_BUS_SPEED, SPI_BR[speed]*1000) 
       == ARM_DRIVER_OK);
          
    /* Read bus speed */
    val = drv->Control(ARM_SPI_GET_BUS_SPEED,0)/1000;
    if (val != SPI_BR[speed]) {
      sprintf(str,"Specific control set speed: %dkHz, Get speed: %dkHz",SPI_BR[speed], val);
      SET_RESULT(WARNING, str);
    } else SET_RESULT(PASSED, NULL);
    
    /* Change bus speed with general control parameter*/
    ASSERT_TRUE(drv->Control(ARM_SPI_MODE_MASTER | ARM_SPI_CPOL0_CPHA0 | ARM_SPI_DATA_BITS(8) | ARM_SPI_MSB_LSB | ARM_SPI_SS_MASTER_SW, SPI_BR[speed]*1000) 
       == ARM_DRIVER_OK);
       
    /* Read bus speed */
    val = drv->Control(ARM_SPI_GET_BUS_SPEED,0)/1000;
    if (val != SPI_BR[speed]) {
      sprintf(str,"General control set speed: %dkHz, Get speed: %dkHz",SPI_BR[speed], val);
      SET_RESULT(WARNING, str);
    } else SET_RESULT(PASSED, NULL);
   
  }
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Send
\details
The test case \b SPI_Send verifies the function \b Send with the sequence:
 - Send using callback
 - Send without callback
*/
void SPI_Send (void) { 
  uint16_t cnt;
  char str[64];
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  
  /* Send data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) { 
    
    /* Send using callback */
    if (SPI_RunSend(buffer_out, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to send %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);   
    
    /* Send without callback */
    if (SPI_RunSend_NoCallback(buffer_out, BUFFER[cnt]) != ARM_DRIVER_OK) {   
      sprintf(str,"Fail to send without callback %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);      
  } 
  
  /* Free buffer */
  free(buffer_out);    
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Receive
\details
The test case \b SPI_Receive verifies the function \b Receive with the sequence:
 - Receive using callback
 - Receive without callback
*/
void SPI_Receive (void) { 
  uint16_t cnt;
  char str[64];
  
  /* Allocate buffer */
  buffer_in = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);
  
  /* Receive data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) { 
    
    /* Receive using callback */
    if (SPI_RunReceive(buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to receive %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);
    
    /* Receive without callback */
    if (SPI_RunReceiveNoCallback(buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to receive without callback %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);       
  } 
  
  /* Free buffer */
  free(buffer_in);    
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Loopback_CheckBusSpeed
\details
The test case \b SPI_Loopback_CheckBusSpeed verifies the function \b Control with the sequence:
 - Change bus speed with specific control parameter
 - Read bus speed
*/
void SPI_Loopback_CheckBusSpeed (void) {
  uint32_t speed, val;          
  uint32_t ticks_measured;
  uint32_t ticks_expected;  
  char str[64];
  double rate;
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER_SIZE_BR*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  buffer_in = (uint8_t*) malloc(BUFFER_SIZE_BR*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);
  
  /* Set Local Loopback */
  SPI_LOCAL_LOOPBACK();
  
  /* Test bus speeds */
  for (speed=0; speed<SPI_BR_NUM; speed++) {
    /* Change bus speed with specific control parameter*/
    ASSERT_TRUE (drv->Control(ARM_SPI_SET_BUS_SPEED, SPI_BR[speed]*1000) 
       == ARM_DRIVER_OK);
          
    /* Read bus speed */
    val = drv->Control(ARM_SPI_GET_BUS_SPEED,0)/1000;
    
    /* Measure transfer time */
    ticks_measured = osKernelSysTick();
    SPI_RunTransfer(buffer_out, buffer_in, BUFFER_SIZE_BR);
    ticks_measured = osKernelSysTick() - ticks_measured;  
    ticks_expected = osKernelSysTickMicroSec(BUFFER_SIZE_BR*1000/val);
    ticks_expected *= SPI_DATA_BITS;
    
    rate = (double)ticks_measured/ticks_expected;
    
    if ((rate>2.0)||(rate<1.0)) {
      sprintf(str,"At %dkHz: measured time is %f x expected time", val, rate);
      SET_RESULT(WARNING, str);
    } else SET_RESULT(PASSED, NULL);     
  }
  
  /* Free buffer */
  free(buffer_out);   
  free(buffer_in); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: SPI_Loopback_Transfer
\details
The test case \b SPI_Loopback_Transfer verifies the function \b Transfer.
*/
void SPI_Loopback_Transfer (void) { 
  uint16_t cnt, i;
  uint8_t pattern[] = BUFFER_PATTERN;
  char str[64];
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  buffer_in = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);

  /* Set Local Loopback */
  SPI_LOCAL_LOOPBACK();
  
  /* Set output buffer pattern*/
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1];) {  
    for (i = 0; i<ARRAY_SIZE(pattern); i++) {
      buffer_out[cnt++] = pattern[i];
    }
  } 

  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer */
    memset(buffer_in,0,BUFFER[cnt]);    
    if (SPI_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 
  
  /* Set output buffer with random data */
  srand(osKernelSysTick());
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1]; cnt++) {  
    buffer_out[cnt] = rand();
  } 

  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer */
    memset(buffer_in,0,BUFFER[cnt]);    
    if (SPI_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 

  
  /* Free buffer */
  free(buffer_out);   
  free(buffer_in);     
}

/**
@}
*/ 
// end of group spi_funcs
