/*-----------------------------------------------------------------------------
 *      Name:         DV_USART.c
 *      Purpose:      USART test cases
 *----------------------------------------------------------------------------
 *      Copyright(c) KEIL - An ARM Company
 *----------------------------------------------------------------------------*/
#include "cmsis_os.h" 
#include "cmsis_dv.h" 
#include "DV_Config.h"
#include "DV_Framework.h"
#include "RTE_Device.h" 
#include "Driver_USART.h"
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 

// USART baudrates
const uint32_t USART_BR[] = {
#if (USART_BR_1>0)
  USART_BR_1,
#endif  
#if (USART_BR_2>0)
  USART_BR_2,
#endif 
#if (USART_BR_3>0)
  USART_BR_3,
#endif 
#if (USART_BR_4>0)
  USART_BR_4,
#endif 
#if (USART_BR_5>0)
  USART_BR_5,
#endif 
#if (USART_BR_6>0)
  USART_BR_6,
#endif 
};
const uint32_t USART_BR_NUM = ARRAY_SIZE(USART_BR);

// Register Driver_USART#
extern ARM_DRIVER_USART CREATE_SYMBOL(Driver_USART, DRV_USART);
static ARM_DRIVER_USART *drv = &CREATE_SYMBOL(Driver_USART, DRV_USART);
static ARM_USART_CAPABILITIES capab;   

// Event flags
static uint8_t volatile Event;  

// USART event
static void USART_DrvEvent (uint32_t event) {
  Event |= event;
}

// USART asynchronous transfer
int8_t USART_RunTransfer (uint8_t *out, uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_USART_EVENT_RECEIVE_COMPLETE;
      
  drv->Receive (in, cnt);
  drv->Send (out, cnt);
    
  tick = osKernelSysTick();
  do {
    if (Event & ARM_USART_EVENT_RECEIVE_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(USART_TRANSFER_TIMEOUT));
  return -1;
}

// USART send
int8_t USART_RunSend (uint8_t *out, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_USART_EVENT_SEND_COMPLETE;
      
  drv->Send (out, cnt);
    
  tick = osKernelSysTick();
  do {
    if (Event & ARM_USART_EVENT_SEND_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(USART_TRANSFER_TIMEOUT));
  return -1;
}

// USART send
int8_t USART_RunSend_NoCallback (uint8_t *out, uint32_t cnt) {
  uint32_t tick;
        
  drv->Send (out, cnt);
    
  tick = osKernelSysTick();
  do {
    if (drv->GetTxCount() == cnt) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(USART_TRANSFER_TIMEOUT));
  return -1;
}

// USART receive
int8_t USART_RunReceive (uint8_t *in, uint32_t cnt) {
  uint32_t tick;
  
  Event &= ~ARM_USART_EVENT_RECEIVE_COMPLETE;
      
  drv->Receive (in, cnt);
    
  tick = osKernelSysTick();
  do {
    if (Event & ARM_USART_EVENT_RECEIVE_COMPLETE) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(USART_TRANSFER_TIMEOUT));
  return -1;
}

// USART receive
int8_t USART_RunReceive_NoCallback (uint8_t *in, uint32_t cnt) {
  uint32_t tick;
        
  drv->Receive (in, cnt);
    
  tick = osKernelSysTick();
  do {
    if (drv->GetRxCount() == cnt) {
      return 0;
    }
  }
  while ((osKernelSysTick() - tick) < osKernelSysTickMicroSec(USART_TRANSFER_TIMEOUT));
  return -1;
}


/*-----------------------------------------------------------------------------
 *      Test cases
 *----------------------------------------------------------------------------*/

 
/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\defgroup usart_funcs USART Validation
\brief USART test cases
\details
The USART validation test performs the following checks:
- API interface compliance.
- Data communication with various transfer sizes and communication parameters.
- Transfer speed of the data communication.
- Loopback communication.

\anchor usart_loop_back_setup
Loopback Communication Setup
----------------------------

To perform loopback communication tests, it is required to connect the USART's \b TX signal to the \b RX signal (refer to the
schematics of your target hardware for detailed pinout information).
@{
*/

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: USART_GetCapabilities
\details
The test case \b USART_GetCapabilities verifies the function \b GetCapabilities.
*/
void USART_GetCapabilities (void) {
  /* Get USART capabilities */
  capab = drv->GetCapabilities();
  ASSERT_TRUE(&capab != NULL); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Initialization
\details
The test case \b USART_Initialization verifies the USART functions with the sequence:
  - \b Initialize without callback
  - \b Uninitialize
  - \b Initialize with callback
*/
void  USART_Initialization (void) {
  
  /* Initialize without callback */
  ASSERT_TRUE(drv->Initialize(NULL) == ARM_DRIVER_OK); 
    
  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Initialize with callback */
  ASSERT_TRUE(drv->Initialize(USART_DrvEvent) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_CheckInvalidInit
\details
The test case \b USART_CheckInvalidInit verifies the driver behaviour when receiving an invalid initialization sequence:
  - \b Uninitialize
  - \b PowerControl with Power off
  - \b PowerControl with Power on
  - \b Control 
*/
void  USART_CheckInvalidInit (void) {
  
  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Try to power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) != ARM_DRIVER_OK); 
  
  /* Try to set configuration */
  ASSERT_TRUE(drv->Control (ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_8 | ARM_USART_PARITY_NONE | 
    ARM_USART_STOP_BITS_1 | ARM_USART_FLOW_CONTROL_NONE, USART_BR[0]) != ARM_DRIVER_OK);
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_PowerControl
\details
The test case \b USART_PowerControl verifies the \b PowerControl function with the sequence:
 - Power on
 - Power low
 - Power off
 - Power on 
*/
void  USART_PowerControl (void) {
  int32_t val;
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);  
    
  /* Power low */
  val = drv->PowerControl (ARM_POWER_LOW);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Low power is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
   
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);
}  

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_PolarityPhase
\details
The test case \b USART_Config_PolarityPhase verifies the \b Control function with the sequence:
 - Set basic SPI bus configuration
 - Change polarity
 - Change phase 
 - Change polarity and phase
*/
void USART_Config_PolarityPhase (void) { 
  
  /* Set basic SPI bus configuration*/
  ASSERT_TRUE(drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_CPOL0 | ARM_USART_CPHA0, USART_BR[0]) == ARM_DRIVER_OK);
  
  /* Change polarity */
  ASSERT_TRUE(drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_CPOL1 | ARM_USART_CPHA0, USART_BR[0]) == ARM_DRIVER_OK);
  
  /* Change phase */
  ASSERT_TRUE(drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_CPOL0 | ARM_USART_CPHA1, USART_BR[0]) == ARM_DRIVER_OK);
  
  /* Change polarity and phase */
  ASSERT_TRUE(drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_CPOL1 | ARM_USART_CPHA1, USART_BR[0]) == ARM_DRIVER_OK);  
}
  
/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_DataBits
\details
The test case \b USART_Config_DataBits verifies the \b Control function with the sequence:
 - Data bits = \token{5}
 - Data bits = \token{6}
 - Data bits = \token{7}
 - Data bits = \token{8}
 - Data bits = \token{9}
*/
void USART_Config_DataBits (void) { 
  int32_t val;  
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_5, USART_BR[0]);
  if (val == ARM_USART_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 5 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_6, USART_BR[0]);
  if (val == ARM_USART_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 6 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_7, USART_BR[0]);
  if (val == ARM_USART_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 7 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_8, USART_BR[0]);
  if (val == ARM_USART_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 8 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_9, USART_BR[0]);
  if (val == ARM_USART_ERROR_DATA_BITS) { SET_RESULT(WARNING, "Data Bits = 9 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }  
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_StopBits
\details
The test case \b USART_Config_StopBits verifies the \b Control function with the sequence:
 - Stop bits = \token{1}
 - Stop bits = \token{2}
 - Stop bits = \token{1.5}
 - Stop bits = \token{0.5}
*/
void USART_Config_StopBits (void) { 
  int32_t val;  
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_STOP_BITS_1, USART_BR[0]);
  if (val == ARM_USART_ERROR_STOP_BITS) { SET_RESULT(WARNING, "Stop Bits = 1 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_STOP_BITS_2, USART_BR[0]);
  if (val == ARM_USART_ERROR_STOP_BITS) { SET_RESULT(WARNING, "Stop Bits = 2 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_STOP_BITS_1_5, USART_BR[0]);
  if (val == ARM_USART_ERROR_STOP_BITS) { SET_RESULT(WARNING, "Stop Bits = 1.5 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_STOP_BITS_0_5, USART_BR[0]);
  if (val == ARM_USART_ERROR_STOP_BITS) { SET_RESULT(WARNING, "Stop Bits = 0.5 are not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }   
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_Parity
\details
The test case \b USART_Config_Parity verifies the \b Control function and sets parity bits with the sequence:
 - even parity
 - no parity
 - odd parity
*/
void USART_Config_Parity (void) { 
  int32_t val;  
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_PARITY_EVEN, USART_BR[0]);
  if (val == ARM_USART_ERROR_PARITY) { SET_RESULT(WARNING, "Even parity is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_PARITY_NONE, USART_BR[0]);
  if (val == ARM_USART_ERROR_PARITY) { SET_RESULT(WARNING, "No parity is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
  
  val = drv->Control(ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_PARITY_ODD, USART_BR[0]);
  if (val == ARM_USART_ERROR_PARITY) { SET_RESULT(WARNING, "Odd parity not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); } 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_Baudrate
\details
The test case \b USART_Config_Baudrate verifies the \b Control function and configures various baudrates.
*/
void USART_Config_Baudrate (void) { 
  uint32_t speed;   
  
  /* Change baud rate*/
  for (speed=0; speed<USART_BR_NUM; speed++) {
    
    ASSERT_TRUE (drv->Control(ARM_USART_MODE_ASYNCHRONOUS, USART_BR[speed]) 
       == ARM_DRIVER_OK);   
  }
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_Baudrate
\details
The test case \b USART_Config_Baudrate verifies the \b Control function and configures various baudrates and measures
the transfer time.
*/
void USART_Loopback_CheckBaudrate (void) { 
  uint32_t speed;          
  uint32_t ticks_measured;
  uint32_t ticks_expected;  
  char str[64];
  double rate;
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER_SIZE_BR*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  buffer_in = (uint8_t*) malloc(BUFFER_SIZE_BR*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);
  
  /* Set Local Loopback */
  USART_LOCAL_LOOPBACK();
  
  /* Change baud rate*/
  for (speed=0; speed<USART_BR_NUM; speed++) {
    
    ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_TX,0) == ARM_DRIVER_OK);
    ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_RX,0) == ARM_DRIVER_OK);
    ASSERT_TRUE(drv->Control(ARM_USART_MODE_ASYNCHRONOUS, USART_BR[speed]) 
       == ARM_DRIVER_OK);       
    ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_TX,1) == ARM_DRIVER_OK);
    ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_RX,1) == ARM_DRIVER_OK);
        
    /* Measure transfer time */
    ticks_measured = osKernelSysTick();
    USART_RunTransfer(buffer_out, buffer_in, BUFFER_SIZE_BR);
    ticks_measured = osKernelSysTick() - ticks_measured;  
    ticks_expected = osKernelSysTickMicroSec(BUFFER_SIZE_BR*1000000/USART_BR[speed]);
    ticks_expected *= USART_DATA_BITS+2;
    
    rate = (double)ticks_measured/ticks_expected;
    
    if ((rate>2.0)||(rate<1.0)) {
      sprintf(str,"At %dbps: measured time is %f x expected time", USART_BR[speed], rate);
      SET_RESULT(WARNING, str);
    } else SET_RESULT(PASSED, NULL);     
  }
  
  /* Free buffer */
  free(buffer_out);   
  free(buffer_in); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Config_CommonParams
\details
The test case \b USART_Config_CommonParams verifies the \b Control function with the sequence:
 - Configure USART bus
 - Set transmitter
 - Set reciever
*/
void  USART_Config_CommonParams (void) {
  
  /* Configure USART bus*/
  ASSERT_TRUE(drv->Control (ARM_USART_MODE_ASYNCHRONOUS | ARM_USART_DATA_BITS_8 | ARM_USART_PARITY_NONE | 
    ARM_USART_STOP_BITS_1 | ARM_USART_FLOW_CONTROL_NONE, USART_BR[0]) == ARM_DRIVER_OK); 
  ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_TX,1) == ARM_DRIVER_OK); 
  ASSERT_TRUE(drv->Control (ARM_USART_CONTROL_RX,1) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Send
\details
The test case \b USART_Send verifies the \b Send function with the sequence:
 - Send data using callback
 - Send data without callback
*/
void USART_Send (void) { 
  uint16_t cnt;
  char str[64];
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  
  /* Send data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) { 
    
    /* Send using callback */
    if (USART_RunSend(buffer_out, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to send %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);   
    
    /* Send without callback */
    if (USART_RunSend_NoCallback(buffer_out, BUFFER[cnt]) != ARM_DRIVER_OK) {   
      sprintf(str,"Fail to send without callback %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);    
  } 
  
  /* Free buffer */
  free(buffer_out);    
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_AsynchronousReceive
\details
The test case \b USART_AsynchronousReceive verifies the \b Receive function with the sequence:
 - Send data using callback
 - Send data without callback
*/
void USART_AsynchronousReceive (void) { 
  uint16_t cnt;
  char str[64];
  
  /* Allocate buffer */
  buffer_in = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  
  /* Send data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) { 
    
    /* Send using callback */
    if (USART_RunReceive(buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to receive %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);   
    
    /* Send without callback */
    if (USART_RunReceive_NoCallback(buffer_out, BUFFER[cnt]) != ARM_DRIVER_OK) {   
      sprintf(str,"Fail to send without callback %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);    
  } 
  
  /* Free buffer */
  free(buffer_in);    
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USART_Loopback_Transfer
\details
The test case \b USART_Loopback_Transfer verifies the \b Transfer function with the sequence:
 - Clear input buffer
 - Transfer data
*/
void USART_Loopback_Transfer (void) { 
  uint16_t cnt, i;
  uint8_t pattern[] = BUFFER_PATTERN;
  char str[64];
  
  /* Allocate buffer */
  buffer_out = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_out != NULL);
  buffer_in = (uint8_t*) malloc(BUFFER[BUFFER_NUM-1]*sizeof(uint8_t));
  ASSERT_TRUE(buffer_in != NULL);
  
  /* Set Local Loopback */
  USART_LOCAL_LOOPBACK()  
                                                
  /* Set output buffer pattern*/
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1];) {  
    for (i = 0; i<ARRAY_SIZE(pattern); i++) {
      buffer_out[cnt++] = pattern[i];
    }
  } 

  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer*/
    memset(buffer_in,0,BUFFER[cnt]);    
    if (USART_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 
  
  /* Set output buffer with random data*/
  srand(osKernelSysTick());
  for (cnt = 0; cnt<BUFFER[BUFFER_NUM-1]; cnt++) {  
    buffer_out[cnt] = rand();
  }   

  /* Transfer data chunks */
  for (cnt = 0; cnt<BUFFER_NUM; cnt++) {      
    /* Clear input buffer*/
    memset(buffer_in,0,BUFFER[cnt]);    
    if (USART_RunTransfer(buffer_out, buffer_in, BUFFER[cnt]) != ARM_DRIVER_OK) {
      sprintf(str,"Fail to transfer block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
    if (memcmp(buffer_in, buffer_out, cnt)!=0) {
      sprintf(str,"Fail to check block of %d bytes",BUFFER[cnt]);
      SET_RESULT(FAILED, str);
    } else SET_RESULT(PASSED, NULL);     
  } 
  
  /* Free buffer */
  free(buffer_out);   
  free(buffer_in);     
}
  
/**
@}
*/ 
// end of group usart_funcs

  

