/*-----------------------------------------------------------------------------
 *      Name:         DV_SPI.c
 *      Purpose:      SPI test cases
 *----------------------------------------------------------------------------
 *      Copyright(c) KEIL - An ARM Company
 *----------------------------------------------------------------------------*/
#include "cmsis_os.h" 
#include "cmsis_dv.h" 
#include "DV_Config.h"
#include "DV_Framework.h"
#include "RTE_Device.h" 
#include "Driver_USBD.h"
#include <stdio.h>
#include <stdlib.h> 
#include <string.h> 

// Register Driver_USBD#
extern ARM_DRIVER_USBD CREATE_SYMBOL(Driver_USBD, DRV_USBD);
static ARM_DRIVER_USBD *drv = &CREATE_SYMBOL(Driver_USBD, DRV_USBD);
static ARM_USBD_CAPABILITIES capab;  

// Event flags
static uint8_t volatile DeviceEvent;  
static uint8_t volatile EndpointEvent; 

// USB Device event
static void USB_DeviceEvent (uint32_t event) {
  DeviceEvent |= event;
}

// USB Endpoint event
static void USB_EndpointEvent (uint8_t endpoint, uint32_t event) {
  EndpointEvent |= event;
}


/*-----------------------------------------------------------------------------
 *      Test cases
 *----------------------------------------------------------------------------*/
 
/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\defgroup usbd_funcs USB Device Validation
\brief USB Device test cases
\details
The USB Device validation test performs the following checks:
- API interface compliance.
- Data communication with various transfer sizes and communication parameters.
- Transfer speed of the data communication.
@{
*/

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief Test case: USBD_GetCapabilities
\details
The test case \b USBD_GetCapabilities verifies the function \b GetCapabilities.
*/
void USBD_GetCapabilities (void) {                    
  /* Get USBD capabilities */
  capab = drv->GetCapabilities();
  ASSERT_TRUE(&capab != NULL); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USBD_Initialization
\details
The test case \b USBD_Initialization verifies the USBD functions with the sequence:
  - \b Initialize without callback
  - \b Uninitialize
  - \b Initialize with callback
*/
void USBD_Initialization (void) { 
    
  /* Initialize without callback */
  ASSERT_TRUE(drv->Initialize(NULL, NULL) == ARM_DRIVER_OK); 
    
  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Initialize with callback */
  ASSERT_TRUE(drv->Initialize(USB_DeviceEvent, USB_EndpointEvent) == ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USBD_CheckInvalidInit
\details
The test case \b USBD_CheckInvalidInit verifies the driver behaviour when receiving an invalid initialization sequence:
  - \b Uninitialize
  - \b PowerControl with Power off
  - \b PowerControl with Power on
*/
void USBD_CheckInvalidInit (void) { 

  /* Uninitialize */
  ASSERT_TRUE(drv->Uninitialize() == ARM_DRIVER_OK); 
  
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Try to power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) != ARM_DRIVER_OK); 
}

/*=======0=========1=========2=========3=========4=========5=========6=========7=========8=========9=========0=========1====*/
/**
\brief  Test case: USBD_PowerControl
\details
The test case \b USBD_PowerControl verifies the \b PowerControl function with the sequence:
 - Power on
 - Power low
 - Power off
 - Power on 
*/
void USBD_PowerControl (void) { 
  int32_t val;
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);  
  
  /* Power low */
  val = drv->PowerControl (ARM_POWER_LOW);
  if (val == ARM_DRIVER_ERROR_UNSUPPORTED) { SET_RESULT(WARNING, "Low power is not supported"); }
  else { ASSERT_TRUE(val == ARM_DRIVER_OK); }
   
  /* Power off */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_OFF) == ARM_DRIVER_OK);
  
  /* Power on */
  ASSERT_TRUE(drv->PowerControl (ARM_POWER_FULL) == ARM_DRIVER_OK);
}

/**
@}
*/ 
// end of group usbd_funcs
